/*
 * Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "system.h"
#include "haptics.h"

#define EN_PIN BIT6
#define PWM_PIN BIT5

static unsigned char index = 0;
static unsigned char timer_counter = 0;
static unsigned char timer_counter_expected = 0;
static const Waveform *pHapticsWaveform = 0;

void haptics_init(void)
{
	P4DIR |= EN_PIN;	// P4.6 Output
	P4OUT &= ~(EN_PIN); // Disable Amplifier

	P4DIR |= PWM_PIN; // P4.5 Output
}

void haptics_send_waveform(const Waveform *pWaveform)
{
	pHapticsWaveform = pWaveform;
	index = 0;

	P4OUT |= EN_PIN; //Enable Amplifier, Start PWM

	if (pWaveform->data[0] == 0x01)
	{
		P4OUT |= PWM_PIN;
	}
	else
	{
		P4OUT &= ~(PWM_PIN);
	}

	timer_counter = 0;
	timer_counter_expected = pWaveform->data[1];

	WDTCTL = WDT_MDLY_0_5; // WDT 4ms, interval timer
	IE1 |= WDTIE;		   // Enable WDT interrupt
}

// Watchdog Timer interrupt service routine
#pragma vector = WDT_VECTOR
__interrupt void watchdog_timer_isr(void)
{
	timer_counter++;

	if (timer_counter == timer_counter_expected)
	{
		timer_counter = 0;

		index += 2;
		if (index < pHapticsWaveform->length)
		{
			P1OUT |= EN_PIN; //Enable Amplifier, Start PWM

			if (pHapticsWaveform->data[index] == 0x01)
			{
				P4OUT |= PWM_PIN;
			}
			else
			{
				P4OUT &= ~(PWM_PIN);
			}

			timer_counter_expected = pHapticsWaveform->data[index + 1];
		}
		else
		{
			P4OUT &= (~EN_PIN);	 // Disable Amplifier
			P4OUT &= ~(PWM_PIN); // PWM Output = LOW

			IE1 &= (~WDTIE);
			WDTCTL = WDTPW + WDTHOLD;
		}
	}
}
