/*
 * Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef UART_H
#define UART_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "system.h"
#include "Touch_GestureEngine.h"

#define FRAME_LABEL_SIGNAL 's'
#define FRAME_LABEL_RAW_DATA 'r'
#define FRAME_LABEL_FINGER_POS 'f'
#define FRAME_LABEL_GESTURE_ID 'g'
#define FRAME_LABEL_MSG 'm'
#define FRAME_LABEL_CALIBRATION 'c'

#define encode_raw_data_frame() encode_frame(FRAME_LABEL_RAW_DATA, 0, data_set.buffer, 16)

	void uart0_init(void);
	void uart0_putchar(uint8 ch);
	void uart0_puts(char *str);
	void uart0_printf(char *fmt, ...);

	void uart1_init(void);
	void uart1_putchar(uint8 ch);
	void uart1_puts(char *str);
	void uart1_printf(char *fmt, ...);

	uint8 encode_frame(uint8 label, uint8 extra_label, uint8 *buffer, uint8 size);
	uint8 encode_finger_position_frame(void);
	uint8 encode_calibration_frame(uint16 data);
	uint8 encode_gesture_frame(uint8 gesture_id);
	uint8 encode_msg_frame(char *str);
	void send_frame(uint8 size);
	const char *get_gesture_string(uint8 gesture_id);

#ifdef __cplusplus
};
#endif

#endif